
classdef Admin % in Admin.m
    methods (Static = true)
        
        
        function [mg] = lnmultigamma(k, a)
            % return logarithm of  multivariate gamma
            mg= (k*(k-1)/4)*log(pi);
            
            for i=1:k
                mg= mg + gammaln(a + (1-i)/2);
            end
            
        end
        
        
        function [se_H,se_S,se_G] = Compute_se_errors_ML(f_handle,theta_opt,T)
            
            % step 1: compute hessian
            H = Admin.hessian_2sided(f_handle,theta_opt);
            inv_H    = inv(H);
            se_H = sqrt(diag(inv_H));
            
            % step 2 Outer product of gradient
            OPG_mat = Admin.Compute_OPG_MV_MODELS(f_handle,theta_opt,T);
            S_G = inv(OPG_mat)/T;
            se_G = sqrt(diag(S_G));
            
            % step 3; build sandwich covariance matrix
            S_COV = T * inv_H * OPG_mat * inv_H;
            se_S = sqrt(diag(S_COV));
        end
        
        function OPG_mat = Compute_OPG_MV_MODELS(f_handle, theta_vec0,T)
            % compute the expected value of the outer product of the gradient
            % associated wit the log-likelihood function (given by
            % f_handle)
            
            k = length(theta_vec0);            
            g_mat = zeros(T,k);
            
            dEps=eps(1);
            dH=dEps^(1/3)*max( abs(theta_vec0), 1e-2);                                   
            
            
            for i = 1:k
                
                theta_vec = theta_vec0;
                
                theta_vec(i) = theta_vec0(i) + dH(i);
                [~,LogLike] = f_handle(theta_vec);
                L_plus = -LogLike;
                
                theta_vec(i) = theta_vec0(i) - dH(i);
                [~,LogLike] = f_handle(theta_vec);
                L_min = -LogLike;
                
                g_mat(:,i) = (L_plus - L_min)/ (2 * dH(i));
                
            end
            
            OPG_mat = zeros(k,k);
            for j = 1:T
                OPG_mat = OPG_mat + g_mat(j,:)'*g_mat(j,:);
            end
            
            OPG_mat = OPG_mat/T;
            
        end
        
        %%
        function H = hessian_2sided(f,x,varargin)
            % PURPOSE:
            %      Computes 2-sided finite difference Hessian
            %
            % USAGE:
            %      H = hessian_2sided(func,x,varargin)
            %
            % INPUTS:
            %      func         - function name, fval = func(x,varargin)
            %      x            - vector of parameters (n x 1)
            %      varargin     - optional arguments passed to the function
            %
            % OUTPUTS:
            %      H            - finite differnce, 2-sided hessian
            %
            % COMMENTS:
            %      Code originally from COMPECON toolbox [www4.ncsu.edu/~pfackler]
            %      documentation modified to fit the format of the Ecoometrics Toolbox
            %      by James P. LeSage, Dept of Economics
            %      University of Toledo
            %      2801 W. Bancroft St,
            %      Toledo, OH 43606
            %      jlesage@spatial-econometrics.com
            %
            % Further modified (to do 2-sided numerical derivs, rather than 1) by:
            % Author: Kevin Sheppard
            % kevin.sheppard@economics.ox.ac.uk
            % Revision: 3    Date: 4/1/2004
            
            try
                feval(f,x,varargin{:});
            catch
                error('There was an error evaluating the function.  Please check the arguements.');
            end
            
            
            n = size(x,1);
            fx = feval(f,x,varargin{:});
            
            % Compute the stepsize (h)
            
            h = eps.^(1/3)*max(abs(x),1e-2);
            xh = x+h;
            h = xh-x;
            ee = sparse(1:n,1:n,h,n,n);
            
            % Compute forward and backward steps
            gp = zeros(n,1);
            for i=1:n
                gp(i) = feval(f,x+ee(:,i),varargin{:});
            end
            gm = zeros(n,1);
            for i=1:n
                gm(i) = feval(f,x-ee(:,i),varargin{:});
            end
            
            
            H=h*h';
            Hm=H;
            Hp=H;
            % Compute "double" forward and backward steps
            for i=1:n
                for j=i:n
                    Hp(i,j) = feval(f,x+ee(:,i)+ee(:,j),varargin{:});
                    Hp(j,i)=Hp(i,j);
                    Hm(i,j) = feval(f,x-ee(:,i)-ee(:,j),varargin{:});
                    Hm(j,i)=Hm(i,j);
                end
            end
            
            %Compute the hessian
            for i=1:n
                for j=i:n
                    H(i,j) = (Hp(i,j)-gp(i)-gp(j)+fx+fx-gm(i)-gm(j)+Hm(i,j))/H(i,j)/2;
                    H(j,i) = H(i,j);
                end
            end
        end
        
        
        
        function [uLT] = UnitLT(k)
            % fill Lower triangular matrix with ones
            uLT= zeros(k,k);
            
            for j=1:k
                for i=j+1:k
                    uLT(i,j) =1;
                end
            end
            
        end
        
        
        
        %%
        function [ A ] = Vech2Sym(k, vechA )
            % this function puts vech of A into A
            
            A= zeros(k,k);
            l=1;
            for j=1:k
                for i=j:k
                    A(i,j)= vechA(l);
                    A(j,i)= A(i,j);
                    l=l+1;
                end
            end
            
        end
        
        function [ A ] = Vec2Sym(k, vecA )
            % this function puts vec of A into A
            
            A= zeros(k,k);
            l=1;
            for j=1:k
                for i=1:k
                    A(i,j)= vecA(l);
                    l=l+1;
                end
            end
            
        end
        
        
        function [ A ] = VecCorr2Sym(k, vecA )
            % this function puts vec of LT mat of A into A
            
            A= zeros(k,k);
            l=1;
            for j=1:k
                for i=j+1:k
                    A(i,j)= vecA(l);
                    l=l+1;
                end
            end
            
        end
        
        
        
        %%
        function [ vechA ] = Sym2Vech(k, A )
            % this function puts A into vech of A
            
            vechA= zeros(k*(k+1)/2, 1);
            l=1;
            for j=1:k
                for i=j:k
                    vechA(l) = A(i,j);
                    l=l+1;
                end
            end
            
        end
        
        %%
        function [ vechltA ] = LowerTr2Vech(k, ltA )
            % this function puts lower triangular A into  vech lt A
            
            vechltA= zeros(k*(k+1)/2, 1);
            l=1;
            for j=1:k
                for i=j:k
                    vechltA(l)= ltA(i,j);
                    l=l+1;
                end
            end
            
        end
        
        function [UtA] =Sym2UT(k,A)
            % this function puts a symmetrix matrix A in to a
            % Upper triangular matrix
            UtA = zeros(k*(k-1)/2,1);
            l = 1;
            for j = 1:k
                UtA(l:l+k-j-1,1) = A(j,j+1:end);
                l =l+k-j;
            end
            
        end
        
        function [A] =Ut2Sym(k,UtA)
            % Upper triangular of A into A with ones on the diagonal
            A = eye(k);
            l = 1;
            for j = 1:k
                A(j,j+1:end) = UtA(l:l+k-j-1,1);
                A(j+1:end,j) =  A(j,j+1:end);
                l =l+k-j;
            end
            
        end
        
        
        
        
        %%
        function [ ltA ] = Vech2LowerTr(k, vechltA )
            % this function puts vector of vech of lt A into lt A
            
            ltA= zeros(k,k);
            l=1;
            for j=1:k
                for i=j:k
                    ltA(i,j)= vechltA(l);
                    l=l+1;
                end
            end
            
        end
        
        %%
        function [K_sp] = commutation(m,  n)
            
            I= eye(m*n, m*n);
            K= zeros(m*n, m*n);
            
            for j=1:n
                for i=1:m
                    K((i-1)*n + j,:)= I((j-1)*m + i, :);
                end
            end
            
            K_sp = sparse(K);
            
        end
        
        
        %%
        function [ D ] = duplication(k)
            % creates duplication matrix
            
            I= eye(k*(k+1)/2);
            D= zeros( k^2, k*(k+1)/2 );
            
            for j=1:k*(k+1)/2
                for i=1:k
                    if (i>=j)
                        D((j-1)*k + i ,:)= I((j-1)*(k - j/2) +i ,:);
                        D((i-1)*k + j ,:)= I((j-1)*(k - j/2) +i ,:);
                    end
                end
            end
            
        end
        
        %%
        function [ L ] = elimination(k)
            % creates elimination matrix
            
            D= Admin.duplication(k);
            L= inv(D' * D) * D';
            
        end
        
        
        %%
        function [DL] = duplication_gen(k)
            
            D = Admin.duplication(k);
            LT= ones(k, k);
            
            for i=1:k
                for j=i+1:k
                    LT(i,j)= 0;
                end
            end
            
            LT= reshape(LT, k^2, 1) * ones(1, k*(k+1)/2 );
            DL= LT .* D;
            
        end
        
        %%
        function [LL] = elimination_gen(k)
            
            DL= Admin.duplication_gen(k);
            LL= DL';
            
        end
    end % end methods
end % end class