function [minus_LLF,loglik_vec_tF,LLF_r2,LLF_RC,loglik_vec_r,loglik_vec_RC,V_mat] = LogLik_GAS_tF_CT(k, T, params, return_mat, RC_mat)

% Date: March - 2022
% This file contain the (MINUS) log-likelihood function of
% the HEAVY GAS tF model from the paper

%  "New HEAVY Models for Fat-Tailed Realized
% Covariances and Returns" by Opschoor et al. (2018).

% NOTE: all references to equations refer to Opschoor et al.(2018)

% General input
%   k: number of assets
%   T: number of time periods
%   return_mat: a matrix of (T by k) open-to-close returns
%   RC_mat: a k by k by T matrix of realized covariances

% General output
%   minus_LLF: the MINUS log likelihood
%   loglike_vec_tF: a (T x 1) vector of the log-likelihood values at time t
%   LLF_r2: the log-likelihood of the t-distribution
%   LLF_RC: the log-likelihood of the Matrix F distribution
%   loglike_vec_r: a (T x 1) vector of t log-likelihood values of returns
%   loglike_vec_RC: a (T x 1) vector of matrix-F log-likelihood values of realized covariance matrices
%   V_mat: fitted covariance matrices


% retrieve parameters
alpha = params(1);
beta  = params(2);
nu_t  = params(3);
nu_f1 = params(4);
nu_f2 = params(5);

% covariance targeting by using the mean of the sample realized covariances
constant = nu_f1/ (nu_f2-k-1);
V0       = mean(RC_mat, 3);
Omega    = (1-beta)*V0;

% set the stage with empty matrices/ starting value of V
V_mat        = zeros(k,k,T);
V_mat(:,:,1) = V0;
I_k          = eye(k);
Flag         = 0; 

loglik_vec_r    = zeros(T,1);
loglik_vec_RC   = zeros(T,1);

for j=1:T
    
    V_j   = V_mat(:,:,j);
    
    % check if V_j is a true positive def covariance matrix
    check_ind = rcond(V_j);
    if check_ind < 1e-15
        Flag = 1;
        break
    end
    
    [~,nr_pd] = chol(V_j);
    if nr_pd>0
        Flag = 1;
        break
    end
    
    % compute V_inverse and r_t r_t'
    invV_j = V_j\I_k;
    RC_j   = RC_mat(:,:,j);
    r2_j   = return_mat(j,:)'*return_mat(j,:);
    
    % compute scaled score w.r.t t distribution
    w_j        = (nu_t + k)/ (nu_t - 2 + trace(invV_j*r2_j));
    nabla_t    =  w_j * r2_j - V_j;
    
    % compute scaled score w.r.t. matrix F distribution
    temp_part_F_dist_j  = I_k + constant*invV_j*RC_j;
    inv_denom_nabla_F   = temp_part_F_dist_j\I_k;
    nabla_F             = nu_f1*((nu_f1 +nu_f2)/(nu_f2-k-1)*RC_j*inv_denom_nabla_F - V_j) ;
    
    % total scaled score
    nabla_TOT = (nabla_t + nabla_F)/(1+nu_f1);
    
    % update V
    if j<T
        V_mat(:,:,j+1) = Omega + beta * V_mat(:,:,j) + alpha * nabla_TOT;
    end
    
    % compute loglikelihood (only those parts that depend on time)
    logdetV_j    = log(det(V_j));
    logdetRC_j   = log(det(RC_j));
    
    % Student T
    loglik_vec_r(j,1) = - 1/2 * logdetV_j  - 0.5*(nu_t + k) * log(1 + (1/(nu_t-2))*trace(invV_j*r2_j));
    % matrix F
    loglik_vec_RC(j,1) = 0.5*nu_f1 * log(det(constant*invV_j)) + 0.5*(nu_f1-k-1)*logdetRC_j - 0.5*(nu_f1+nu_f2)*log(det(temp_part_F_dist_j));
end

% complete log-likelihood with time-invariant parts
loglik_vec_r = loglik_vec_r  + gammaln(0.5*(nu_t + k)) - gammaln(nu_t/2) - 0.5*k*log((nu_t-2)*pi);
loglik_vec_RC = loglik_vec_RC + Admin.lnmultigamma(k, 0.5*(nu_f1+nu_f2)) - Admin.lnmultigamma(k, 0.5*nu_f1) - Admin.lnmultigamma(k, 0.5*nu_f2);

LLF_r2 = sum(loglik_vec_r);
LLF_RC = sum(loglik_vec_RC);

loglik_vec_tF = loglik_vec_r + loglik_vec_RC;
minus_LLF= -sum(loglik_vec_tF);

if  isnan(minus_LLF) || Flag==1
    minus_LLF=1e14;
end

end






